@@ -16,14 +16,14 @@
 #
 # keltus <keltus@users.sourceforge.net>
 #
-# $Id: irclib.py,v 1.37 2005/08/18 20:11:22 keltus Exp $
+# $Id: irclib.py,v 1.43 2005/12/24 22:12:40 keltus Exp $
 
 """irclib -- Internet Relay Chat (IRC) protocol client library.
 
 This library is intended to encapsulate the IRC protocol at a quite
 low level.  It provides an event-driven IRC client framework.  It has
-a fairly thorough support for the basic IRC protocol and CTCP, but DCC
-connection support is not yet implemented.
+a fairly thorough support for the basic IRC protocol, CTCP, DCC chat,
+but DCC file transfers is not yet supported.
 
 In order to understand how to make an IRC client, I'm afraid you more
 or less must understand the IRC specifications.  They are available
@@ -54,14 +54,10 @@ Current limitations:
   * The IRC protocol shines through the abstraction a bit too much.
   * Data is not written asynchronously to the server, i.e. the write()
     may block if the TCP buffers are stuffed.
-  * There are no support for DCC connections.
+  * There are no support for DCC file transfers.
   * The author haven't even read RFC 2810, 2811, 2812 and 2813.
   * Like most projects, documentation is lacking...
 
-Since I seldom use IRC anymore, I will probably not work much on the
-library.  If you want to help or continue developing the library,
-please contact me (Joel Rosdahl <joel@rosdahl.net>).
-
 .. [IRC specifications] http://www.irchelp.org/irchelp/rfc/
 """
 
@@ -74,7 +70,7 @@ import sys
 import time
 import types
 
-VERSION = 0, 4, 5
+VERSION = 0, 4, 6
 DEBUG = 0
 
 # TODO
@@ -259,7 +255,7 @@ class IRC:
         \"NO MORE\", no more handlers will be called.
         """
 
-        if not self.handlers.has_key(event):
+        if not event in self.handlers:
             self.handlers[event] = []
         bisect.insort(self.handlers[event], ((priority, handler)))
 
@@ -274,7 +270,7 @@ class IRC:
 
         Returns 1 on success, otherwise 0.
         """
-        if not self.handlers.has_key(event):
+        if not event in self.handlers:
             return 0
         for h in self.handlers[event]:
             if handler == h[1]:
@@ -521,16 +517,16 @@ class ServerConnection(Connection):
                     self.real_server_name = prefix
 
             if m.group("command"):
-                command = string.lower(m.group("command"))
+                command = m.group("command").lower()
 
             if m.group("argument"):
-                a = string.split(m.group("argument"), " :", 1)
-                arguments = string.split(a[0])
+                a = m.group("argument").split(" :", 1)
+                arguments = a[0].split()
                 if len(a) == 2:
                     arguments.append(a[1])
 
             # Translate numerics into more readable strings.
-            if numeric_events.has_key(command):
+            if command in numeric_events:
                 command = numeric_events[command]
 
             if command == "nick":
@@ -567,7 +563,6 @@ class ServerConnection(Connection):
                                 command, prefix, target, m)
                         self._handle_event(Event(command, prefix, target, m))
                         if command == "ctcp" and m[0] == "ACTION":
-                            # Emit an action event too. We're generous today.
                             self._handle_event(Event("action", prefix, target, m[1:]))
                     else:
                         if DEBUG:
@@ -578,7 +573,7 @@ class ServerConnection(Connection):
                 target = None
 
                 if command == "quit":
-                    arguments = arguments[0]
+                    arguments = [arguments[0]]
                 elif command == "ping":
                     target = arguments[0]
                 else:
@@ -597,7 +592,7 @@ class ServerConnection(Connection):
     def _handle_event(self, event):
         """[Internal]"""
         self.irclibobj._handle_event(self, event)
-        if self.handlers.has_key(event.eventtype()):
+        if event.eventtype() in self.handlers:
             for fn in self.handlers[event.eventtype()]:
                 fn(self, event)
 
@@ -628,11 +623,11 @@ class ServerConnection(Connection):
 
     def admin(self, server=""):
         """Send an ADMIN command."""
-        self.send_raw(string.strip(string.join(["ADMIN", server])))
+        self.send_raw(" ".join(["ADMIN", server]).strip())
 
     def ctcp(self, ctcptype, target, parameter=""):
         """Send a CTCP command."""
-        ctcptype = string.upper(ctcptype)
+        ctcptype = ctcptype.upper()
         self.privmsg(target, "\001%s%s\001" % (ctcptype, parameter and (" " + parameter) or ""))
 
     def ctcp_reply(self, target, parameter):
@@ -649,9 +644,10 @@ class ServerConnection(Connection):
         if not self.connected:
             return
 
+        self.connected = 0
+
         self.quit(message)
 
-        self.connected = 0
         try:
             self.socket.close()
         except socket.error, x:
@@ -665,11 +661,11 @@ class ServerConnection(Connection):
 
     def info(self, server=""):
         """Send an INFO command."""
-        self.send_raw(string.strip(string.join(["INFO", server])))
+        self.send_raw(" ".join(["INFO", server]).strip())
 
     def invite(self, nick, channel):
         """Send an INVITE command."""
-        self.send_raw(string.strip(string.join(["INVITE", nick, channel])))
+        self.send_raw(" ".join(["INVITE", nick, channel]).strip())
 
     def ison(self, nicks):
         """Send an ISON command.
@@ -678,7 +674,7 @@ class ServerConnection(Connection):
 
             nicks -- List of nicks.
         """
-        self.send_raw("ISON " + string.join(nicks, " "))
+        self.send_raw("ISON " + " ".join(nicks))
 
     def join(self, channel, key=""):
         """Send a JOIN command."""
@@ -701,7 +697,7 @@ class ServerConnection(Connection):
         """Send a LIST command."""
         command = "LIST"
         if channels:
-            command = command + " " + string.join(channels, ",")
+            command = command + " " + ",".join(channels)
         if server:
             command = command + " " + server
         self.send_raw(command)
@@ -720,7 +716,7 @@ class ServerConnection(Connection):
 
     def names(self, channels=None):
         """Send a NAMES command."""
-        self.send_raw("NAMES" + (channels and (" " + string.join(channels, ",")) or ""))
+        self.send_raw("NAMES" + (channels and (" " + ",".join(channels)) or ""))
 
     def nick(self, newnick):
         """Send a NICK command."""
@@ -740,7 +736,7 @@ class ServerConnection(Connection):
         if type(channels) == types.StringType:
             self.send_raw("PART " + channels + (message and (" " + message)))
         else:
-            self.send_raw("PART " + string.join(channels, ",") + (message and (" " + message)))
+            self.send_raw("PART " + ",".join(channels) + (message and (" " + message)))
 
     def pass_(self, password):
         """Send a PASS command."""
@@ -762,7 +758,7 @@ class ServerConnection(Connection):
     def privmsg_many(self, targets, text):
         """Send a PRIVMSG command to multiple targets."""
         # Should limit len(text) here!
-        self.send_raw("PRIVMSG %s :%s" % (string.join(targets, ","), text))
+        self.send_raw("PRIVMSG %s :%s" % (",".join(targets), text))
 
     def quit(self, message=""):
         """Send a QUIT command."""
@@ -805,7 +801,7 @@ class ServerConnection(Connection):
 
     def topic(self, channel, new_topic=None):
         """Send a TOPIC command."""
-        if new_topic == None:
+        if new_topic is None:
             self.send_raw("TOPIC " + channel)
         else:
             self.send_raw("TOPIC %s :%s" % (channel, new_topic))
@@ -820,7 +816,7 @@ class ServerConnection(Connection):
 
     def userhost(self, nicks):
         """Send a USERHOST command."""
-        self.send_raw("USERHOST " + string.join(nicks, ","))
+        self.send_raw("USERHOST " + ",".join(nicks))
 
     def users(self, server=""):
         """Send a USERS command."""
@@ -840,7 +836,7 @@ class ServerConnection(Connection):
 
     def whois(self, targets):
         """Send a WHOIS command."""
-        self.send_raw("WHOIS " + string.join(targets, ","))
+        self.send_raw("WHOIS " + ",".join(targets))
 
     def whowas(self, nick, max="", server=""):
         """Send a WHOWAS command."""
@@ -1167,19 +1163,18 @@ def mask_matches(nick, mask):
     """
     nick = irc_lower(nick)
     mask = irc_lower(mask)
-    mask = string.replace(mask, "\\", "\\\\")
+    mask = mask.replace("\\", "\\\\")
     for ch in ".$|[](){}+":
-        mask = string.replace(mask, ch, "\\" + ch)
-    mask = string.replace(mask, "?", ".")
-    mask = string.replace(mask, "*", ".*")
+        mask = mask.replace(ch, "\\" + ch)
+    mask = mask.replace("?", ".")
+    mask = mask.replace("*", ".*")
     r = re.compile(mask, re.IGNORECASE)
     return r.match(nick)
 
-_alpha = "abcdefghijklmnopqrstuvwxyz"
 _special = "-[]\\`^{}"
-nick_characters = _alpha + string.upper(_alpha) + string.digits + _special
-_ircstring_translation = string.maketrans(string.upper(_alpha) + "[]\\^",
-                                          _alpha + "{}|~")
+nick_characters = string.ascii_letters + string.digits + _special
+_ircstring_translation = string.maketrans(string.ascii_uppercase + "[]\\^",
+                                          string.ascii_lowercase + "{}|~")
 
 def irc_lower(s):
     """Returns a lowercased string.
@@ -1187,7 +1182,7 @@ def irc_lower(s):
     The definition of lowercased comes from the IRC specification (RFC
     1459).
     """
-    return string.translate(s, _ircstring_translation)
+    return s.translate(_ircstring_translation)
 
 def _ctcp_dequote(message):
     """[Internal] Dequote a message according to CTCP specifications.
@@ -1219,7 +1214,7 @@ def _ctcp_dequote(message):
     else:
         # Split it into parts.  (Does any IRC client actually *use*
         # CTCP stacking like this?)
-        chunks = string.split(message, _CTCP_DELIMITER)
+        chunks = message.split(_CTCP_DELIMITER)
 
         messages = []
         i = 0
@@ -1230,7 +1225,7 @@ def _ctcp_dequote(message):
 
             if i < len(chunks)-2:
                 # Aye!  CTCP tagged data ahead!
-                messages.append(tuple(string.split(chunks[i+1], " ", 1)))
+                messages.append(tuple(chunks[i+1].split(" ", 1)))
 
             i = i + 2
 
@@ -1257,13 +1252,13 @@ def ip_numstr_to_quad(num):
     n = long(num)
     p = map(str, map(int, [n >> 24 & 0xFF, n >> 16 & 0xFF,
                            n >> 8 & 0xFF, n & 0xFF]))
-    return string.join(p, ".")
+    return ".".join(p)
 
 def ip_quad_to_numstr(quad):
     """Convert an IP address string (e.g. '192.168.0.1') to an IP
     number as an integer given in ASCII representation
     (e.g. '3232235521')."""
-    p = map(long, string.split(quad, "."))
+    p = map(long, quad.split("."))
     s = str((p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3])
     if s[-1] == "L":
         s = s[:-1]
@@ -1274,29 +1269,29 @@ def nm_to_n(s):
 
     (The source of an Event is a nickmask.)
     """
-    return string.split(s, "!")[0]
+    return s.split("!")[0]
 
 def nm_to_uh(s):
     """Get the userhost part of a nickmask.
 
     (The source of an Event is a nickmask.)
     """
-    return string.split(s, "!")[1]
+    return s.split("!")[1]
 
 def nm_to_h(s):
     """Get the host part of a nickmask.
 
     (The source of an Event is a nickmask.)
     """
-    return string.split(s, "@")[1]
+    return s.split("@")[1]
 
 def nm_to_u(s):
     """Get the user part of a nickmask.
 
     (The source of an Event is a nickmask.)
     """
-    s = string.split(s, "!")[1]
-    return string.split(s, "@")[0]
+    s = s.split("!")[1]
+    return s.split("@")[0]
 
 def parse_nick_modes(mode_string):
     """Parse a nick mode string.
@@ -1336,7 +1331,7 @@ def _parse_modes(mode_string, unary_modes=""):
     # State variable.
     sign = ""
 
-    a = string.split(mode_string)
+    a = mode_string.split()
     if len(a) == 0:
         return []
     else:
@@ -1436,7 +1431,7 @@ numeric_events = {
     "324": "channelmodeis",
     "329": "channelcreate",
     "331": "notopic",
-    "332": "topic",
+    "332": "currenttopic",
     "333": "topicinfo",
     "341": "inviting",
     "342": "summoning",
